'use strict';

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _index = require('../index');

var _index2 = _interopRequireDefault(_index);

var _helper = require('./helper');

var _helper2 = _interopRequireDefault(_helper);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ERROR_ID = 'form.errors.inclusion';

function test(value, params) {
  return (0, _helper2.default)((0, _index.inclusion)(params)(value));
}

describe('Validator: inclusion', function () {
  it('should be invalid when `value` is not in the list', function () {
    _assert2.default.equal(ERROR_ID, test(1, { in: [] }));
    _assert2.default.equal(ERROR_ID, test('1', { in: [2, 3, 4] }));
    _assert2.default.equal(ERROR_ID, test('foo', { within: 'foobar' }));
    _assert2.default.equal(ERROR_ID, test('FOO', { within: ['foobar'], caseSensitive: false }));
    _assert2.default.equal(ERROR_ID, test('FOO', { within: ['foo'], caseSensitive: true }));
  });
  it('should be valid when `value` is in the list', function () {
    _assert2.default.ok(!test(1, { in: [9, 8, '1'] }));
    _assert2.default.ok(!test('1', { in: [9, 8, 1] }));
    _assert2.default.ok(!test('foo', { within: 'foo' }));
    _assert2.default.ok(!test('foo', { within: ['bar', 'foo'], caseSensitive: true }));
    _assert2.default.ok(!test('FOO', { within: ['foo'], caseSensitive: false }));
    _assert2.default.ok(!test('fOo', { within: ['foo'], caseSensitive: false }));
  });
  it('should use default caseSensitive option', function () {
    var defaultValue = _index2.default.defaultOptions.caseSensitive;

    _index2.default.defaultOptions.caseSensitive = true;
    _assert2.default.ok(!test('foo', { within: 'foo' }));

    _index2.default.defaultOptions.caseSensitive = false;
    _assert2.default.ok(!test('foo', { within: 'FOO' }));

    _index2.default.defaultOptions.caseSensitive = defaultValue;
  });
  it('should use formatMessage', function () {
    var defaultValue = _index2.default.formatMessage;

    _index2.default.formatMessage = function (msg) {
      return Object.assign({}, msg, { id: msg.id + '2' });
    };
    _assert2.default.equal(ERROR_ID + '2', test('1', { in: [2, 3, 4] }));

    _index2.default.formatMessage = defaultValue;
  });
});